@@ -1,8 +1,8 @@
-import os
-from typing import List
+from typing import List, Union
 import requests, json
 from ddd_objects.infrastructure.ao import exception_class_dec
 from ddd_objects.infrastructure.repository_impl import error_factory
+from ddd_objects.domain.exception import return_codes
 from .do import (
     CommandResultDO,
     ConditionDO,
@@ -10,6 +10,8 @@ from .do import (
     ConfigMapUserSettingDO,
     DeploymentDO,
     IngressDO,
+    InstanceInfoDO,
+    InstanceTypeWithStatusDO,
     NamespaceDO,
     NodeInfoDO,
     NodeMetaDO, 
@@ -29,8 +31,12 @@ class K3SController:
 
     def _check_error(self, status_code, info):
         if status_code>299:
-            return_code = info['detail']['return_code']
-            error_info = info['detail']['error_info']
+            if isinstance(info['detail'], str):
+                return_code = return_codes['OTHER_CODE']
+                error_info = info['detail']
+            else:
+                return_code = info['detail']['return_code']
+                error_info = info['detail']['error_info']
             raise error_factory.make(return_code)(error_info)
 
     @exception_class_dec(max_try=1)
@@ -56,7 +62,7 @@ class K3SController:
         return NodeInfoDO(**info)
 
     @exception_class_dec(max_try=1)
-    def delete_nodes(self, node_infos: List[NodeInfoDO], timeout=60):
+    def delete_nodes(self, node_infos: List[Union[NodeInfoDO, NodeMetaDO]], timeout=60):
         data = [info.to_json() for info in node_infos]
         data = json.dumps(data)
         response=requests.delete(f'{self.url}/nodes', headers=self.header, data=data, timeout=timeout)
@@ -64,6 +70,18 @@ class K3SController:
         self._check_error(response.status_code, info)
         return info
 
+    @exception_class_dec(max_try=1)
+    def get_instance_info_by_node_meta(self, region_id: str, node_meta: NodeMetaDO, timeout=300):
+        data = json.dumps(node_meta.to_json())
+        response=requests.get(f'{self.url}/instance/region_id/{region_id}/node_meta', 
+            headers=self.header, data=data, timeout=timeout)
+        info = json.loads(response.text)
+        self._check_error(response.status_code, info)
+        instance_info, instance_type_info = info
+        instance_info = InstanceInfoDO(**instance_info)
+        instance_type_info = InstanceTypeWithStatusDO(**instance_type_info)
+        return instance_info, instance_type_info
+
     @exception_class_dec(max_try=1)
     def get_existing_nodes(self, cluster_name: str, timeout=300):
         response=requests.get(f'{self.url}/nodes/cluster_name/{cluster_name}', 
@@ -272,47 +290,51 @@ repo_info = \
         "get_config_maps":{
             "args":[["cluster_name", False, "Name"], ["namespace_name", False, "Name"]],
             "ret":["config_map", True, None, None, True, True],
-            "key": "{cluster_name}:{namespace_name}:config_maps"
+            "key": "{cluster_name.get_value()}:{namespace_name.get_value()}:config_maps"
         },
         "get_deployments":{
             "args":[["cluster_name", False, "Name"], ["namespace_name", False, "Name"]],
             "ret":["deployment", True, None, None, True, True],
-            "key": "{cluster_name}:{namespace_name}:deployments"
+            "key": "{cluster_name.get_value()}:{namespace_name.get_value()}:deployments"
+        },
+        "get_instance_info_by_node_meta":{
+            "args":[["region_id", False], ["node_meta"]],
+            "key": "{region_id.get_value()}:{node_name.get_value()}:{node_ip.get_value}:instance_and_instance_type_info"
         },
         "get_existing_nodes":{
             "args":[["cluster_name", False, "Name"]],
             "ret":["node_info", True, None, None, True, True],
-            "key": "{cluster_name}:existing_nodes"
+            "key": "{cluster_name.get_value()}:existing_nodes"
         },
         "get_existing_nodes_by_name":{
             "args":[["node_name", False, "Name"]],
             "ret":["node_info", True, None, None, True, True],
-            "key": "{node_name}:existing_nodes"
+            "key": "{node_name.get_value()}:existing_nodes"
         },
         "get_namespaces":{
             "args":[["cluster_name", False, "Name"]],
             "ret":["namespace", True, None, None, True, True],
-            "key": "{cluster_name}:namespaces"
+            "key": "{cluster_name.get_value()}:namespaces"
         },
         "get_ingresses":{
             "args":[["cluster_name", False, "Name"], ["namespace_name", False, "Name"]],
             "ret":["ingress", True, None, None, True, True],
-            "key": "{cluster_name}:{namespace_name}:ingresses"
+            "key": "{cluster_name.get_value()}:{namespace_name.get_value()}:ingresses"
         },
         "get_node_metas":{
             "args":[["cluster_name", False, "Name"]],
             "ret":["node_meta", True, None, None, True, True],
-            "key": "{cluster_name}:node_metas"
+            "key": "{cluster_name.get_value()}:node_metas"
         },
         "get_pods":{
             "args":[["cluster_name", False, "Name"], ["namespace_name", False, "Name"]],
             "ret":["pod", True, None, None, True, True],
-            "key": "{cluster_name}:{namespace_name}:pods"
+            "key": "{cluster_name.get_value()}:{namespace_name.get_value()}:pods"
         },
         "get_secrets":{
             "args":[["cluster_name", False, "Name"], ["namespace_name", False, "Name"]],
             "ret":["secret", True, None, None, True, True],
-            "key": "{cluster_name}:{namespace_name}:secrets"
+            "key": "{cluster_name.get_value()}:{namespace_name.get_value()}:secrets"
         },
         "upload_to_oss_from_pod":{
             "args":[["pod_oss_operation_info", True, "PodOSSOperationInfo", None, False, False, "pod_oss_operation_info_converter"]]
